import os
import sys
import traceback

import network
import path
import utils
   
IS_MISSING = -1

def approxEqual(value, target, tolerance):
   if (abs(target) <= tolerance ): return abs(value) <= tolerance
   return abs(float(value) / target - 1) <= tolerance
   
def check(name, value, target, tolerance):
   if approxEqual(value, target, tolerance):
      return True
   else:
      print("\nWrong %s: your value %s, correct value %s"
               % (name, value, target))
      return False

def linkCounting(testFileName):

   print("Running link counting test: " + str(testFileName) + "...", end='')
   
   try:
      with open(testFileName, "r") as testFile:
         # Read test information
         try:
            fileLines = testFile.read().splitlines()
            pointsPossible = IS_MISSING
            networkFile = IS_MISSING
            tripsFile = IS_MISSING
            for line in fileLines:
               # Ignore comments and blank lines
               if len(line.strip()) == 0 or line[0] == '#':
                  continue
                   
               # Set points possible
               if pointsPossible == IS_MISSING:
                  pointsPossible = int(line)
                  continue
                  
               # Identify network files
               if networkFile == IS_MISSING:
                  networkFile = os.path.normpath(line)
                  continue
               if tripsFile == IS_MISSING:
                  tripsFile = os.path.normpath(line)
                  continue
                  
               # Set problem parameters
               answer = int(line)
                  
         except:
            print("\nError running test %s, attempting to continue with remaining tests.  Exception details: " % testFileName)
            traceback.print_exc(file=sys.stdout)
            return 0, 0
            
         # Now run the actual test
         try:
            testNetwork = network.Network(networkFile, tripsFile)
            testNode = testNetwork.findLeastEnteringLinks()
            correct = check("node ", testNode, answer, 0)
            if correct == False:               
               print("...fail")
               return 0, pointsPossible
         except utils.NotYetAttemptedException:
            print("...not yet attempted")
            return 0, pointsPossible
         except:
            print("\nException raised, attempting to continue:")
            traceback.print_exc(file=sys.stdout)                     
            print("\n...fail")
            return 0, pointsPossible
            
         print("...pass")
         return pointsPossible, pointsPossible

         
   except IOError:
      print("\nError running test %s, attempting to continue with remaining tests.  Exception details: " % testFileName)
      traceback.print_exc(file=sys.stdout) 
      return 0, 0
   

def matrix(testFileName):

   print("Running adjacency matrix test: " + str(testFileName) + "...", end='')
   
   answer = list()
   
   try:
      with open(testFileName, "r") as testFile:
         # Read test information
         try:
            fileLines = testFile.read().splitlines()
            pointsPossible = IS_MISSING
            networkFile = IS_MISSING
            tripsFile = IS_MISSING
            for line in fileLines:
               # Ignore comments and blank lines
               if len(line.strip()) == 0 or line[0] == '#':
                  continue
                   
               # Set points possible
               if pointsPossible == IS_MISSING:
                  pointsPossible = int(line)
                  continue
                  
               # Identify network files
               if networkFile == IS_MISSING:
                  networkFile = os.path.normpath(line)
                  continue
               if tripsFile == IS_MISSING:
                  tripsFile = os.path.normpath(line)
                  continue
                  
               # Read another line in answer
               answer.append(line.split())
                  
         except:
            print("\nError running test %s, attempting to continue with remaining tests.  Exception details: " % testFileName)
            traceback.print_exc(file=sys.stdout)
            return 0, 0
            
         # Now run the actual test
         try:
            testNetwork = network.Network(networkFile, tripsFile)
            testNetwork.formAdjacencyMatrix()
            correct = True
            for i in testNetwork.node:
               for j in testNetwork.node:
                  correct = correct and check("matrix value %s %s" % (i,j), testNetwork.adjacencyMatrix[i][j], int(answer[int(i)-1][int(j)-1]), 0)
            if correct == False:               
               print("...fail")
               return 0, pointsPossible
         except utils.NotYetAttemptedException:
            print("...not yet attempted")
            return 0, pointsPossible
         except:
            print("\nException raised, attempting to continue:")
            traceback.print_exc(file=sys.stdout)                     
            print("\n...fail")
            return 0, pointsPossible
            
         print("...pass")
         return pointsPossible, pointsPossible

         
   except IOError:
      print("\nError running test %s, attempting to continue with remaining tests.  Exception details: " % testFileName)
      traceback.print_exc(file=sys.stdout) 
      return 0, 0
   

def topological(testFileName):

   print("Running topological order test: " + str(testFileName) + "...", end='')
   
   answer = list()
   
   try:
      with open(testFileName, "r") as testFile:
         # Read test information
         try:
            fileLines = testFile.read().splitlines()
            pointsPossible = IS_MISSING
            networkFile = IS_MISSING
            tripsFile = IS_MISSING
            for line in fileLines:
               # Ignore comments and blank lines
               if len(line.strip()) == 0 or line[0] == '#':
                  continue
                   
               # Set points possible
               if pointsPossible == IS_MISSING:
                  pointsPossible = int(line)
                  continue
                  
               # Identify network files
               if networkFile == IS_MISSING:
                  networkFile = os.path.normpath(line)
                  continue
               if tripsFile == IS_MISSING:
                  tripsFile = os.path.normpath(line)
                  continue
                  
         except:
            print("\nError running test %s, attempting to continue with remaining tests.  Exception details: " % testFileName)
            traceback.print_exc(file=sys.stdout)
            return 0, 0
            
         # Now run the actual test
         try:
            testNetwork = network.Network(networkFile, tripsFile)
            testNetwork.findTopologicalOrder()
            correct = True
            for ij in testNetwork.link:
               check = testNetwork.node[testNetwork.link[ij].tail].order < testNetwork.node[testNetwork.link[ij].head].order
               if check == False:
                  print("Bad topological order: node %s should precede node %s, your order is %s and %s" % (testNetwork.link[ij].tail, testNetwork.link[ij].head, testNetwork.node[testNetwork.link[ij].tail].order, testNetwork.node[testNetwork.link[ij].head].order))
               correct = correct and check
            if correct == False:               
               print("...fail")
               return 0, pointsPossible
         except utils.NotYetAttemptedException:
            print("...not yet attempted")
            return 0, pointsPossible
         except:
            print("\nException raised, attempting to continue:")
            traceback.print_exc(file=sys.stdout)                     
            print("\n...fail")
            return 0, pointsPossible
            
         print("...pass")
         return pointsPossible, pointsPossible

         
   except IOError:
      print("\nError running test %s, attempting to continue with remaining tests.  Exception details: " % testFileName)
      traceback.print_exc(file=sys.stdout) 
      return 0, 0
   


def networkLoading(testFileName):

   print("Running network loading test: " + str(testFileName) + "...", end='')

   answer = list()
      
   try:
      with open(testFileName, "r") as testFile:
         # Read test information
         try:
            fileLines = testFile.read().splitlines()
            pointsPossible = IS_MISSING
            networkFile = IS_MISSING
            tripsFile = IS_MISSING
            for line in fileLines:
               # Ignore comments and blank lines
               if len(line.strip()) == 0 or line[0] == '#':
                  continue
                   
               # Set points possible
               if pointsPossible == IS_MISSING:
                  pointsPossible = int(line)
                  continue
                  
               # Identify network files
               if networkFile == IS_MISSING:
                  networkFile = os.path.normpath(line)
                  continue
               if tripsFile == IS_MISSING:
                  tripsFile = os.path.normpath(line)
                  continue

               # Read another line in answer
               answer.append(line.split())
                  
         except:
            print("\nError running test %s, attempting to continue with remaining tests.  Exception details: " % testFileName)
            traceback.print_exc(file=sys.stdout)
            return 0, 0
            
         # Now run the actual test
         try:
            testNetwork = network.Network(networkFile, tripsFile)
            for p in answer:
               pathTuple = utils.path2linkTuple(p[0])
               testNetwork.path[p[0]] = path.Path(pathTuple, testNetwork, float(p[1]))
            testNetwork.loadPaths()
            correct = True
            for p in answer:
               correct = correct and check("path %s travel time" % p[0], testNetwork.path[p[0]].cost, float(p[2]), 0.01)
            if correct == False:               
               print("...fail")
               return 0, pointsPossible
         except utils.NotYetAttemptedException:
            print("...not yet attempted")
            return 0, pointsPossible
         except:
            print("\nException raised, attempting to continue:")
            traceback.print_exc(file=sys.stdout)                     
            print("\n...fail")
            return 0, pointsPossible
            
         print("...pass")
         return pointsPossible, pointsPossible

         
   except IOError:
      print("\nError running test %s, attempting to continue with remaining tests.  Exception details: " % testFileName)
      traceback.print_exc(file=sys.stdout) 
      return 0, 0
   
